#include <iostream>
#include<boost/graph/adjacency_list.hpp>
#include <boost/graph/filtered_graph.hpp>

using namespace boost;

template <typename EdgeWeightMap>
struct positive_edge_weight {

  positive_edge_weight() {}

  positive_edge_weight(EdgeWeightMap weight) : m_weight(weight) {}

  template <typename Edge>
  bool operator()(const Edge& e) const {
    return 0 < get(m_weight, e);
  }

  EdgeWeightMap m_weight;
};

int main(int argc, char ** argv) {

  typedef adjacency_list<vecS, vecS, directedS, no_property, property<edge_weight_t, int> > Graph;
  typedef property_map<Graph, edge_weight_t>::type EdgeWeightMap;
  typedef filtered_graph<Graph, positive_edge_weight<EdgeWeightMap> > FilteredGraph;
  graph_traits < Graph >::vertex_iterator vi, vi_end;
  graph_traits < Graph >::out_edge_iterator oei, oei_end;
  graph_traits < FilteredGraph >::vertex_iterator fvi, fvi_end;
  graph_traits < FilteredGraph >::out_edge_iterator foei, foei_end;

  enum { A, B, C, D, E, N };
  const char* name = "ABCDE";
  Graph g(N);
  add_edge(A, B, 2, g);
  add_edge(A, C, 0, g);
  add_edge(C, D, 1, g);
  add_edge(D, E, 0, g);
  add_edge(D, B, 3, g);
  add_edge(E, C, 0, g);

  std::cout<<"Graph out-edges:"<<std::endl;
  for (tie(vi, vi_end) = vertices(g); vi != vi_end; ++vi) {
    std::cout<<name[*vi]<<" outedges - ";
    for(tie(oei, oei_end)=out_edges(*vi, g); oei != oei_end; ++oei) {
      std::cout<<name[source(*oei, g)]<<"-->"<<name[target(*oei, g)]<<", ";
    }
    std::cout<<std::endl;
  }
  std::cout<<std::endl;

  positive_edge_weight<EdgeWeightMap> filter(get(edge_weight, g));

  FilteredGraph fg(g, filter);

  std::cout<<"Filtered graph out-edges:"<<std::endl;
  for (tie(fvi, fvi_end) = vertices(fg); fvi != fvi_end; ++fvi) {
    std::cout<<name[*fvi]<<" outedges - ";
    for(tie(foei, foei_end)=out_edges(*fvi, fg); foei != foei_end; ++foei) {
      std::cout<<name[source(*foei, fg)]<<"-->"<<name[target(*foei, fg)]<<", ";
    }
    std::cout<<std::endl;
  }

  return EXIT_SUCCESS;
}
