#include <iostream>
#include <fstream>
#include <boost/lexical_cast.hpp>
#include <boost/graph/graphviz.hpp>
#include <boost/graph/kruskal_min_spanning_tree.hpp>

namespace boost {
  enum graph_color_t { graph_color = 5556 };
  BOOST_INSTALL_PROPERTY(graph, color);
}

int main(int argc, char ** argv) {

  using namespace boost;

  typedef 
    adjacency_list<vecS, vecS, undirectedS,
                   property<vertex_name_t, std::string>, 
                   property<edge_color_t, std::string,
                            property<edge_weight_t, int> >,
                   property<graph_color_t, std::string> >
    g_dot_type;
  g_dot_type g_dot;

  dynamic_properties dp(ignore_other_properties);
  dp.property("node_id", get(vertex_name, g_dot));
  dp.property("label", get(edge_weight, g_dot));
  dp.property("color", get(edge_color, g_dot));
  dp.property("color", ref_property_map<g_dot_type*, std::string>(get_property(g_dot, graph_color)));

  std::ifstream infile("input/telephone-network.dot");
  read_graphviz(infile, g_dot, dp);

  typedef adjacency_list < vecS, vecS, undirectedS, no_property,
    property < edge_weight_t, int > > Graph;

  Graph g(num_vertices(g_dot));
  graph_traits < g_dot_type >::edge_iterator ei, ei_end;
  for (tie(ei, ei_end) = edges(g_dot); ei != ei_end; ++ei) {
    int weight = get(edge_weight, g_dot, *ei);
    property < edge_weight_t, int >edge_property(weight);
    add_edge(source(*ei, g_dot), target(*ei, g_dot), edge_property, g);
  }

  std::vector<graph_traits<Graph>::edge_descriptor> mst;
  kruskal_minimum_spanning_tree(g, std::back_inserter(mst));

  property_map<Graph, edge_weight_t>::type weight = get(edge_weight, g);
  int total_weight = 0;
  for(int i=0; i<mst.size(); ++i) total_weight += get(weight, mst[i]);
  std::cout << "total weight: " << total_weight << std::endl;

  for(tie(ei, ei_end) = edges(g_dot); ei != ei_end; ++ei) put(edge_color, g_dot, *ei, "grey");
  
  for(int i=0; i<mst.size(); ++i) {
    graph_traits < Graph >::vertex_descriptor u = source(mst[i], g);
    graph_traits < Graph >::vertex_descriptor v = target(mst[i], g);
    graph_traits < Graph >::edge_descriptor e = edge(u, v, g_dot).first;
    put(edge_color, g_dot, e, "black");
  }

  std::ofstream outfile("output/kruskal-telephone-out.dot");
  write_graphviz_dp(outfile, g_dot, dp);

  return EXIT_SUCCESS;
}
