#include <iostream>
#include <boost/array.hpp>
#include <boost/graph/edge_list.hpp>
#include <boost/graph/bellman_ford_shortest_paths.hpp>

int main(int argc, char ** argv) {
  using namespace boost;

  // set-up router network
  enum {A, B, C, D, E, F, G, H, n_vertices };
  const int n_edges = 11;
  typedef std::pair<int, int> Edge;
  array<Edge, n_edges> edges = {{ Edge(A,B), Edge(A,C), Edge(B,D), Edge(B,E),
				  Edge(C,E), Edge(C,F), Edge(D,H), Edge(D,E),
				  Edge(E,H), Edge(F,G), Edge(G,H) }};
  typedef edge_list<array<Edge, n_edges>::iterator> Graph;
  Graph g(edges.begin(),edges.end());

  // assign edge weights
  array<float, n_edges> delay = {{ 5.0, 1.0, 1.3, 3.0, 10.0, 2.0, 6.3, 0.4, 1.3, 1.2, 0.5 }};

  // create vertex property storage
  char name[] = "ABCDEFGH";
  array<int, n_vertices> parent;
  for(int i=0; i<n_vertices; ++i) parent[i] = i;
  array<float, n_vertices> distance;
  distance.assign(std::numeric_limits<float>::max());
  distance[A] = 0;

  // call bellman-ford algorithm
  bool r = bellman_ford_shortest_paths(g,
				       int(n_vertices),
				       weight_map(make_iterator_property_map(delay.begin(),
									     get(edge_index, g),
									     delay[0])).
				       distance_map(&distance[0]).
				       predecessor_map(&parent[0]));

  // output distances and parents
  if(r) {
    for(int i=0; i<n_vertices; ++i) {
      std::cout<<name[i]<<": "<<distance[i]<<" "<<name[parent[i]]<<std::endl;
    }
  }
  else {
    std::cout<<"negative cycle"<<std::endl;
  }

  return EXIT_SUCCESS;
}
