//#include <iostream>
#include <fstream>
#include <map>
#include <string>
#include <boost/graph/strong_components.hpp>
#include <boost/graph/graphviz.hpp>

namespace boost {
  enum graph_color_t { graph_color = 5556 };
  BOOST_INSTALL_PROPERTY(graph, color);
}

int main(int argc, char ** argv) {

  using namespace boost;

  typedef 
    adjacency_list<vecS, vecS, directedS,
                   property<vertex_name_t, std::string,
                            property<vertex_color_t, std::string> >,
                   no_property,
                   property<graph_color_t, std::string> >
    g_dot_type;
  g_dot_type g_dot;

  dynamic_properties dp(ignore_other_properties);
  dp.property("node_id", get(vertex_name, g_dot));
  dp.property("color", get(vertex_color, g_dot));
  dp.property("color", ref_property_map<g_dot_type*, std::string>(get_property(g_dot, graph_color)));

  std::ifstream infile("input/scc.dot");
  read_graphviz(infile, g_dot, dp);

  typedef graph_traits<g_dot_type>::vertex_descriptor Vertex;
  std::map<Vertex, int> component;
  int num_comp = strong_components(g_dot, make_assoc_property_map(component));
  
  std::string color[] = {"white", "grey", "black", "lightgrey"};
  graph_traits<g_dot_type>::vertex_iterator vi, vi_end;
  for (tie(vi, vi_end) = vertices(g_dot); vi != vi_end; ++vi) {
    put(vertex_color, g_dot, *vi, color[component[*vi]]);
    std::cout << get(vertex_name, g_dot, *vi) << " " << color[component[*vi]] << std::endl;
  }  

  std::ofstream outfile("output/scc-out.dot");
  write_graphviz_dp(outfile, g_dot, dp);

  return EXIT_SUCCESS;
}
